package com.diagbot.aggregate;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.facade.NeoFacade;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.BillNeoVO;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 从图谱去开单数据聚合
 * @author: gaodm
 * @time: 2020/10/22 9:24
 */
@Component
@Slf4j
public class BillNeoAggregate {
    @Autowired
    private NeoFacade neoFacade;

    @DataProvider("getBillNeo")
    public List<BillNeoDTO> getBillNeo(
            @InvokeParameter("billNeoVO") BillNeoVO billNeoVO,
            @DataConsumer("getDrugBill") List<BillNeoDTO> drugBillList,
            @DataConsumer("getPacsBill") List<BillNeoDTO> pacsBillList,
            @DataConsumer("getLisDetailBill") List<BillNeoDTO> lisDetailBillList,
            @DataConsumer("getLisBill") List<BillNeoDTO> lisBillList,
            @DataConsumer("getOperationBill") List<BillNeoDTO> operationBillList,
            @DataConsumer("getCombiOpBill") List<BillNeoDTO> combiOpBillList,
            @DataConsumer("getTransfusionBill") List<BillNeoDTO> transfusionBillList) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        addbillNeoDTO(billNeoDTOs, drugBillList);
        addbillNeoDTO(billNeoDTOs, pacsBillList);
        addbillNeoDTO(billNeoDTOs, lisDetailBillList);
        addbillNeoDTO(billNeoDTOs, lisBillList);
        addbillNeoDTO(billNeoDTOs, operationBillList);
        addbillNeoDTO(billNeoDTOs, combiOpBillList);
        addbillNeoDTO(billNeoDTOs, transfusionBillList);

        return billNeoDTOs;
    }

    private void addbillNeoDTO(List<BillNeoDTO> billNeoDTOs, List<BillNeoDTO> billNeoDTOList) {
        if (ListUtil.isNotEmpty(billNeoDTOList)) {
            billNeoDTOs.addAll(billNeoDTOList);
        }
    }

    /**
     * 药品
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getDrugBill")
    public List<BillNeoDTO> getDrugBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getDrugBill(billNeoVO.getDrugList());
    }

    /**
     * 检查
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getPacsBill")
    public List<BillNeoDTO> getPacsBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getPacsBill(billNeoVO.getPacsList());
    }

    /**
     * 检验明细
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getLisDetailBill")
    public List<BillNeoDTO> getLisDetailBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getLisDetailBill(billNeoVO.getLisDetailList());
    }

    /**
     * 检验套餐
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getLisBill")
    public List<BillNeoDTO> getLisBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getLisBill(billNeoVO.getLisList());
    }

    /**
     * 手术和操作
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getOperationBill")
    public List<BillNeoDTO> getOperationBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getOperationBill(billNeoVO.getOperationList());
    }

    /**
     * 联合项目
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getCombiOpBill")
    public List<BillNeoDTO> getCombiOpBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getCombiOpBill(billNeoVO.getCombiopList());
    }

    /**
     * 输血
     *
     * @param billNeoVO
     * @return
     */
    @DataProvider("getTransfusionBill")
    public List<BillNeoDTO> getTransfusionBill(@InvokeParameter("billNeoVO") BillNeoVO billNeoVO) {
        return neoFacade.getTransfusionBill(billNeoVO.getTransfusionList());
    }
}
