package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.process.BillProcess;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.Drug;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.ItemExt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 开单合理性facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    BillProcess billProcess;
    @Autowired
    CommonFacade commonFacade;

    /**
     * 开单合理性业务
     * @param
     * @return
     */
    public void billFac(IndicationPushVO indicationPushVO, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 开单合理性图谱接口调用
        BillNeoVO billNeoVO = fillBillNeo(wordCrfDTO);
        List<BillNeoDTO> billNeoDTOList = neoFacade.getBillNeo(billNeoVO);

        // 规则处理
        billProcess.process(billNeoDTOList, wordCrfDTO, res);

        // 结果去重处理
        commonFacade.dealMsg(res.getBillMsgList());
    }

    public BillNeoVO fillBillNeo(WordCrfDTO wordCrfDTO) {
        BillNeoVO billNeoVO = new BillNeoVO();

        List<Drug> drugs = wordCrfDTO.getDrugOrder();

        for (Drug item : drugs) {
            billNeoVO.getDrugList().add(new HashMap<String, String>()
                        {{put(item.getName(), item.getUniqueName());}});
        }

        List<Pacs> pacslist = wordCrfDTO.getPacsOrder();
        for (Pacs pacs : pacslist) {
            billNeoVO.getPacsList().add(new HashMap<String, String>()
                        {{put(pacs.getName(), pacs.getUniqueName());}});
        }

        List<Lis> lislist = wordCrfDTO.getLisOrder();
        for (Lis lis : lislist) {
            if (StringUtil.isBlank(lis.getDetailName())) {
                billNeoVO.getLisList().add(new HashMap<String, String>()
                        {{put(lis.getName(), lis.getUniqueName());}});
            }
            else {
                billNeoVO.getLisDetailList().add(new HashMap<String, String>()
                        {{put(lis.getName(), lis.getUniqueName());}});
            }
        }

        List<Item> operations = wordCrfDTO.getOperationOrder();
        for (Item item : operations) {
            billNeoVO.getOperationList().add(new HashMap<String, String>()
                        {{put(item.getName(), item.getUniqueName());}});
        }

        List<Item> diags = wordCrfDTO.getDiagOrder();
        for (Item item : diags) {
            billNeoVO.getDiagList().add(new HashMap<String, String>()
                        {{put(item.getName(), item.getUniqueName());}});
        }

        List<ItemExt> transfusions = wordCrfDTO.getTransfusionOrder();
        for (ItemExt item : transfusions) {
            billNeoVO.getTransfusionList().add(new HashMap<String, String>()
                        {{put(item.getName(), item.getUniqueName());}});
        }
        return billNeoVO;
    }
}
