package com.diagbot.facade;

import com.diagbot.dto.DiseaseInfoDTO;
import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.entity.BaseNodeInfo;
import com.diagbot.entity.node.LisName;
import com.diagbot.entity.node.LisSet;
import com.diagbot.entity.node.Medicine;
import com.diagbot.entity.node.YiBaoDiseaseName;
import com.diagbot.entity.node.YiBaoOperationName;
import com.diagbot.entity.relationship.LisNameLisSet;
import com.diagbot.enums.StaticTypeEnum;
import com.diagbot.repository.BaseNodeRepository;
import com.diagbot.repository.DeptNode;
import com.diagbot.repository.DeptRepository;
import com.diagbot.repository.LisNameNode;
import com.diagbot.repository.LisNameRepository;
import com.diagbot.repository.LisSetNode;
import com.diagbot.repository.LisSetRepository;
import com.diagbot.repository.MedicineNode;
import com.diagbot.repository.MedicineRepository;
import com.diagbot.repository.PacsNameNode;
import com.diagbot.repository.PacsNameRepository;
import com.diagbot.repository.PacsSubNameRepository;
import com.diagbot.repository.TransfusionRemindRepository;
import com.diagbot.repository.YiBaoDiseaseNode;
import com.diagbot.repository.YiBaoDiseaseRepository;
import com.diagbot.repository.YiBaoOperationNameNode;
import com.diagbot.repository.YiBaoOperationNameRepository;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.NeoUtil;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/8/12 16:44
 */
@Component
public class RetrievalFacade {

    @Autowired
    LisSetRepository lisSetRepository;

    @Autowired
    LisNameRepository lisRepository;

    @Autowired
    YiBaoDiseaseRepository icdDiseaseRepository;

    @Autowired
    PacsNameRepository pacsRepository;

    @Autowired
    PacsSubNameRepository pacsSubNameRepository;

    @Autowired
    MedicineRepository medicineRepository;

    @Autowired
    YiBaoOperationNameRepository operationRepository;

    @Autowired
    TransfusionRemindRepository transfusionRemindRepository;

    @Autowired
    BaseNodeRepository baseNodeRepository;

    @Autowired
    DeptRepository deptRepository;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        RetrievalDTO retrievalDTO = new RetrievalDTO();
        switch (retrievalVO.getType()) {
            case 1:
                retrievalDTO.setLisNames(getLisPacks(retrievalVO.getInputStr()));
                break;
            case 2:
                retrievalDTO.setLisDetailNames(getLis(retrievalVO.getInputStr()));
                break;
            case 3:
                retrievalDTO.setPacsNames(getPACSs(retrievalVO.getInputStr()));
                break;
            case 4:
                retrievalDTO.setDiseaseNames(getDiseases(retrievalVO.getInputStr()));
                break;
            case 5:
                retrievalDTO.setDrugNames(getMedicines(retrievalVO.getInputStr()));
                break;
            case 6:
                retrievalDTO.setOperationNames(getOperations(retrievalVO.getInputStr()));
                break;
            case 7:
                retrievalDTO.setDeptNames(getDepts(retrievalVO.getInputStr()));
                break;
            case 8:
                retrievalDTO.setTransfusionNames(getTransfusion(retrievalVO.getInputStr()));
                break;
        }

        return retrievalDTO;
    }


    /*
     * 获取化验套餐名称列表
     */
    public List<String> getLisPacks(String info) {

        LisSetNode lisPackNode = new LisSetNode();
        List<String> lisPacklist = lisPackNode.getLISPackNameList(info, lisSetRepository, baseNodeRepository);

        return lisPacklist;
    }


    /*
     * 获取化验细项名称列表
     */
    public List<LisDetailDTO> getLis(String info) {

        LisNameNode lisNode = new LisNameNode();
        List<LisDetailDTO> lislist = lisNode.getLisDetailList(info, lisRepository, baseNodeRepository);

        return lislist;
    }


    /*
     * 获取辅检名称列表
     */
    public List<String> getPACSs(String info) {

        PacsNameNode pacsNode = new PacsNameNode();
        List<String> pacslist = pacsNode.getPACSNameAndPACSSubNameList(info, pacsRepository);

        return pacslist;
    }


    /*
     * 获取疾病名称列表
     */
    public List<DiseaseInfoDTO> getDiseases(String info) {

        YiBaoDiseaseNode yiBaoDiseaseNode = new YiBaoDiseaseNode();
        List<DiseaseInfoDTO> diseaselist = yiBaoDiseaseNode.getDiseaseNameList(info, icdDiseaseRepository, baseNodeRepository);

        return diseaselist;
    }


    /*
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicines(String info) {

        MedicineNode medicineNode = new MedicineNode();
        List<DrugInfoDTO> medslist = medicineNode.getMedicineList(info, medicineRepository, baseNodeRepository);

        return medslist;
    }


    /*
     * 获取手术和操作名称列表
     */
    public List<OperationInfoDTO> getOperations(String info) {

        YiBaoOperationNameNode operationNode = new YiBaoOperationNameNode();
        List<OperationInfoDTO> opslist = operationNode.getOperationList(info, operationRepository, baseNodeRepository);

        return opslist;
    }


    /*
     * 获取输血方式列表
     */
    public List<String> getTransfusion(String info) {
        List<String> transfusionNames = transfusionRemindRepository.findTransfusionByName(info);

        return transfusionNames;
    }


    /*
     * 获取科室名称列表
     */
    public List<String> getDepts(String name) {

        DeptNode deptNode = new DeptNode();
        List<String> deptlist = deptNode.getDeptNameList(name, deptRepository, baseNodeRepository);

        return deptlist;
    }


    /**
     * 医学知识（静态信息）检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        String inputStr = staticKnowledgeIndexVO.getInputStr();
        List<Integer> types = staticKnowledgeIndexVO.getTypes();
        List<YiBaoDiseaseName> icdDiseaseList = Lists.newLinkedList();
        List<Medicine> medicineList = Lists.newLinkedList();
        List<LisSet> lisSetList = Lists.newLinkedList();
        List<LisName> lisList = Lists.newLinkedList();
        List<BaseNodeInfo> pacsList = Lists.newLinkedList();
        List<YiBaoOperationName> operationList = Lists.newLinkedList();

        //检索类型（多选）：0-全部、1-诊断、2-药品、3-检验、4-检查、5-手术和操作
        int size = 100;
        if (ListUtil.isEmpty(types)
                || (ListUtil.isNotEmpty(types) && types.contains(0))) {
            icdDiseaseList = icdDiseaseRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            size -= icdDiseaseList.size();
            if (size > 0) {
                medicineList = medicineRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                size -= medicineList.size();
            }
            if (size > 0) {
                lisSetList = lisSetRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                size -= lisSetList.size();
            }
            if (size > 0) {
                lisList = lisRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                size -= lisList.size();
            }
            if (size > 0) {
                pacsList = pacsRepository.staticKnowledgeIndexMultipleLabel(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                size -= pacsList.size();
            }
            if (size > 0) {
                operationList = operationRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                size -= operationList.size();
            }
        } else {
            if (types.contains(1)) {
                icdDiseaseList = icdDiseaseRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            }
            if (types.contains(2)) {
                medicineList = medicineRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            }
            if (types.contains(3)) {
                lisSetList = lisSetRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
                lisList = lisRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            }
            if (types.contains(5)) {
                pacsList = pacsRepository.staticKnowledgeIndexMultipleLabel(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            }
            if (types.contains(6)) {
                operationList = operationRepository.staticKnowledgeIndex(staticKnowledgeIndexVO.getHasInfo(), inputStr, inputStr, size);
            }
        }

        //TODO 诊断
        if (ListUtil.isNotEmpty(icdDiseaseList)) {
            for (YiBaoDiseaseName icdDisease : icdDiseaseList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(icdDisease.getName());
                staticDTO.setType(StaticTypeEnum.DIS.getKey());
                staticDTO.setTypeName(StaticTypeEnum.DIS.getName());
                retList.add(staticDTO);
            }
        }
        //TODO 药品
        if (ListUtil.isNotEmpty(medicineList)) {
            for (Medicine medicine : medicineList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(medicine.getName());
                staticDTO.setType(StaticTypeEnum.DRUG.getKey());
                staticDTO.setTypeName(StaticTypeEnum.DRUG.getName());
                retList.add(staticDTO);
            }
        }
        //TODO 检验套餐
        List<StaticKnowledgeIndexDTO> lisSetRetList = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(lisSetList)) {
            for (LisSet lisSet : lisSetList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(lisSet.getName());
                staticDTO.setType(StaticTypeEnum.LISSET.getKey());
                staticDTO.setTypeName(StaticTypeEnum.LISSET.getName());
                lisSetRetList.add(staticDTO);
            }
        }
        //TODO 检验明细关联出检验套餐
        List<StaticKnowledgeIndexDTO> lisSetRelatecLisRetList = Lists.newLinkedList();
        //TODO 检验明细
        List<StaticKnowledgeIndexDTO> lisRetList = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(lisList)) {
            for (LisName lis : lisList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(lis.getName());
                staticDTO.setType(StaticTypeEnum.LIS.getKey());
                staticDTO.setTypeName(StaticTypeEnum.LIS.getName());
                lisRetList.add(staticDTO);
                if (lis.getLisNamelissets().size() > 0) {
                    for (LisNameLisSet lisSet : lis.getLisNamelissets()) {
                        if (NeoUtil.isExist(lisSet)) {
                            StaticKnowledgeIndexDTO staticLisSetDTO = new StaticKnowledgeIndexDTO();
                            staticLisSetDTO.setRetrievalName(lis.getName());
                            staticLisSetDTO.setName(lisSet.getLisSet().getName());
                            staticLisSetDTO.setType(StaticTypeEnum.LISSET.getKey());
                            staticLisSetDTO.setTypeName(StaticTypeEnum.LISSET.getName());
                            lisSetRelatecLisRetList.add(staticLisSetDTO);
                        }
                    }
                }
            }
        }
        //合并检验套餐
        Map<String, List<StaticKnowledgeIndexDTO>> lisSetMap
                = EntityUtil.makeEntityListMap(lisSetRetList, "name");
        Map<String, List<StaticKnowledgeIndexDTO>> lisSetRelatecLisMap
                = EntityUtil.makeEntityListMap(lisSetRelatecLisRetList, "name");
        for (Map.Entry<String, List<StaticKnowledgeIndexDTO>> entry : lisSetRelatecLisMap.entrySet()) {
            if (lisSetMap == null
                    || lisSetMap.size() == 0
                    || !lisSetMap.containsKey(entry.getKey())) {
                lisSetRetList.add(entry.getValue().get(0));
            }
        }
        //去重后加入检索结果集
        if (ListUtil.isNotEmpty(lisSetRetList)) {
            retList.addAll(lisSetRetList);
        }
        //检验细项加在套餐之后
        if (ListUtil.isNotEmpty(lisRetList)) {
            retList.addAll(lisRetList);
        }

        //TODO 检查
        if (ListUtil.isNotEmpty(pacsList)) {
            for (BaseNodeInfo pacs : pacsList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(pacs.getName());
                if (pacs.getLabels() != null && pacs.getLabels().size() > 0) {
                    if (pacs.getLabels().contains(StaticTypeEnum.PACS.getName())) {
                        staticDTO.setType(StaticTypeEnum.PACS.getKey());
                        staticDTO.setTypeName(StaticTypeEnum.PACS.getName());
                    } else if (pacs.getLabels().contains(StaticTypeEnum.PACSSUBNAME.getName())) {
                        staticDTO.setType(StaticTypeEnum.PACSSUBNAME.getKey());
                        staticDTO.setTypeName(StaticTypeEnum.PACSSUBNAME.getName());
                    }
                } else {
                    staticDTO.setType(StaticTypeEnum.PACS.getKey());
                    staticDTO.setTypeName(StaticTypeEnum.PACS.getName());
                }
                retList.add(staticDTO);
            }
        }

        //TODO 手术和操作
        if (ListUtil.isNotEmpty(operationList)) {
            for (YiBaoOperationName operation : operationList) {
                StaticKnowledgeIndexDTO staticDTO = new StaticKnowledgeIndexDTO();
                staticDTO.setName(operation.getName());
                staticDTO.setType(StaticTypeEnum.OPT.getKey());
                staticDTO.setTypeName(StaticTypeEnum.OPT.getName());
                retList.add(staticDTO);
            }
        }
        if (ListUtil.isNotEmpty(retList) && retList.size() > 100) {
            retList = retList.subList(0, 100);
        }
        return retList;
    }
}