package com.diagbot.model.ai.process;

import com.alibaba.fastjson.JSONObject;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.model.ai.model.EntityEnum;
import com.diagbot.model.ai.model.Lemma;
import com.diagbot.model.entity.BodyPart;
import com.diagbot.model.entity.Modification;
import com.diagbot.model.entity.Pacs;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class EntityProcessPacs extends EntityProcess {

    public void extractEntity(JSONObject aiOut, PacsLabel pacsLabel) {
        // 获取描述和部位、修饰的关系
        List<Pacs> pacsResults = new ArrayList<>(); // 辅检提取的描述信息
        List<Lemma> pacsLemmas = createEntityTree(aiOut, EntityEnum.AUXILIARY_DESCRIPT.toString());
        for (Lemma lemma : pacsLemmas) {
            Pacs pacs = new Pacs();
            List<BodyPart> bodyPartList = new ArrayList<>();
            List<Modification> modificationList = new ArrayList<>();
            if (lemma.isHaveChildren()) {
                for (Lemma relationLemma : lemma.getRelationLemmas()) {
                    if (relationLemma.getProperty().equals(EntityEnum.BODY.toString())) { // 部位
                        BodyPart bodyPart = new BodyPart();
                        bodyPart.setName(relationLemma.getText());
                        bodyPart.setStandName(relationLemma.getText());
                        bodyPartList.add(bodyPart);
                    } else if (EntityEnum.MODIFICATION.toString().equals(relationLemma.getProperty())) { // 修饰
                        Modification modification = new Modification();
                        modification.setName(relationLemma.getText());
                        modification.setStandName(relationLemma.getText());
                        modificationList.add(modification);
                    }
                }
            }
            pacs.setBodyPartList(bodyPartList);
            pacs.setModification(modificationList);
            pacs.setName(lemma.getText());
            pacs.setStandName(lemma.getText());
            pacsResults.add(pacs);
        }
        pacsLabel.setPacsResults(pacsResults);

        // 辅检项目
        List<Lemma> pacsAll = createEntityTree(aiOut, EntityEnum.AUXILIARY_EXAMINATION.toString());
        List<Item> collect = Lists.newArrayList();
        if(ListUtil.isNotEmpty(pacsAll)) {
            collect = pacsAll.stream().map(x -> {
                Item item = new Item();
                item.setName(x.getText());
                item.setUniqueName(x.getText());
                return item;
            }).collect(Collectors.toList());
        }
        pacsLabel.setItem(collect);

        // 辅检诊断
        List<Lemma> pacsDisease = createEntityTree(aiOut, EntityEnum.DIEASE.toString());
        List<Pacs> disList = new ArrayList<>();
        for (Lemma lemma : pacsDisease) {
            Pacs dis = new Pacs();
            dis.setName(lemma.getText());
            dis.setStandName(lemma.getText());
            disList.add(dis);
        }
        pacsLabel.setDisease(disList);
    }
}
