package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.entity.relationship.*;
import com.diagbot.util.NeoUtil;
import com.diagbot.vo.Drug;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;


public class MedRegNameNode {

    /**
     * 获取高危药品数据
     */
    public List<HighRiskNeoDTO> getHighRiskDrug(List<Drug> druglist, MedicineRepository medRepository,
                                                MedicineCodeRepository medCodeRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String drugname, drugform;
        List<Medicine> medicine;
        List<MedCodeName> medcdnames;
        MedRegName medreg;

        for (Drug drug : druglist) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            drugname = drug.getUniqueName();
            highRiskNeoDTO.setName(drug.getName());
            highRiskNeoDTO.setStandname(drugname);
            drugform = drug.getForm();

            medicine = medRepository.findByNameIs(drugname);

            if (null != medicine) {
                for (Medicine med: medicine) {
                    for (MedCodeNameMedicine medCodeMed : med.getMedcodenameMedicines()) {
                        if (NeoUtil.isExist(medCodeMed)) {
                            MedCodeName medcdname = medCodeMed.getMedCodeName();
                            if (medcdname.getReg_JiXin().equals(drugform)) {
                                medcdnames = medCodeRepository.findByNameIs(medcdname.getName());
                                for (MedCodeName medcdnm : medcdnames) {
                                    for (MedCodeNameMedRegName medCodeMedReg : medcdnm.getMedCodeNameMedregnames()) {
                                        if (NeoUtil.isExist(medCodeMedReg)) {
                                            medreg = medCodeMedReg.getMedRegName();
                                            if (null != medreg.getRisklevel() && medreg.getRisklevel().trim().length() > 0) {
                                                highRiskNeoDTO.setIsHighrisk(true);
                                                highRiskNeoDTO.setHighLevel(medreg.getRisklevel());
                                                highRiskNeoDTO.setType(Constants.yaoping);
                                                highRiskNeoDTO.setForm(drugform);
                                                highRiskNeoDTOS.add(highRiskNeoDTO);
                                                break;
                                            }
                                        }
                                    }
                                }

                                if (highRiskNeoDTO.getType() != null) {
                                    break;
                                }
                            }
                        }
                    }

                    if (highRiskNeoDTO.getType()!=null) {
                        break;
                    }

                }
            }
        }

        return highRiskNeoDTOS;
    }


    public DrugBillNeoDTO MedRegNametoDrugDTO(MedRegName medRegName, DrugBillNeoDTO drugBillNeoDTO) {

        Set<MedRegNameVital> medVitals = medRegName.getMedRegNameVital();
        for (MedRegNameVital medvital : medVitals) {
            if (NeoUtil.isExist(medvital)) {
                NodeNeoDTO nodeNeoDTO = NeoUtil.fillNodeNeoDTO(medvital.getVital());
                drugBillNeoDTO.setAgeNeoDTO(nodeNeoDTO);
            }
        }


        Set<MedRegNameGroup> medGroups = medRegName.getMedRegNameGroup();
        for (MedRegNameGroup medgroup : medGroups) {
            if (NeoUtil.isExist(medgroup) && !NeoUtil.inlist(medgroup.getGroup().getName(),drugBillNeoDTO.getGroup())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medgroup.getGroup().getName());
                nodeNeoDTO.setTermtype(Constants.group);
                drugBillNeoDTO.getGroup().add(nodeNeoDTO);
            }
        }


        Set<MedRegNameMedAllergen> medAllergens = medRegName.getMedRegNameMedAllergens();
        for (MedRegNameMedAllergen medAllergen : medAllergens) {
            if (NeoUtil.isExist(medAllergen) && !NeoUtil.inlist(medAllergen.getMedAllergen().getName(), drugBillNeoDTO.getAllergicmeds())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medAllergen.getMedAllergen().getName());
                nodeNeoDTO.setTermtype(medAllergen.getMedAllergen().getMedtype());
                drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
            }
        }


        Set<MedRegNameMedRegName> medRegmedRegNames = medRegName.getMedRegNameMedregnames_out();
        for (MedRegNameMedRegName medRegnm : medRegmedRegNames) {
            if (NeoUtil.isExist(medRegnm) && !NeoUtil.inlist(medRegnm.getMedRegName().getName(), drugBillNeoDTO.getAllergicmeds())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medRegnm.getMedRegName().getName());
                nodeNeoDTO.setTermtype(Constants.yaoping);
                drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
            }
        }


        String conflict_gender = medRegName.getConflict_gender();
        if (null!=conflict_gender) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(conflict_gender);
            nodeNeoDTO.setTermtype(Constants.gender);
            drugBillNeoDTO.setGender(nodeNeoDTO);
        }


        String conflict_geiyao = medRegName.getConflict_geiyao();
        if (null!=conflict_geiyao) {
            NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
            nodeNeoDTO.setName(conflict_geiyao);
            nodeNeoDTO.setTermtype(Constants.geiyao);
            drugBillNeoDTO.setGeiyaoNeoDTO(nodeNeoDTO);
        }


        return drugBillNeoDTO;
    }
}

