package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.entity.relationship.MedicineDisease;
import com.diagbot.entity.relationship.MedicineGroup;
import com.diagbot.entity.relationship.MedicineMedAllergen;
import com.diagbot.entity.relationship.MedicineVital;
import com.diagbot.util.ListUtil;
import com.diagbot.util.NeoUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MedicineNode {


    public DrugBillNeoDTO MedicinetoDrugDTO(Medicine medicine) {
        DrugBillNeoDTO drugBillNeoDTO = new DrugBillNeoDTO();

        Set<MedicineVital> vitals = medicine.getMedicineVital();
        for (MedicineVital vital : vitals) {
            if (NeoUtil.isExist(vital)) {
                NodeNeoDTO nodeNeoDTO = NeoUtil.fillNodeNeoDTO(vital.getVital());
                drugBillNeoDTO.setAgeNeoDTO(nodeNeoDTO);
            }
        }

        Set<MedicineGroup> groups = medicine.getMedicineGroup();
        for (MedicineGroup group : groups) {
            if (NeoUtil.isExist(group) && !NeoUtil.inlist(group.getGroup().getName(),drugBillNeoDTO.getGroup())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(group.getGroup().getName());
                nodeNeoDTO.setTermtype(Constants.group);
                drugBillNeoDTO.getGroup().add(nodeNeoDTO);
            }
        }


        Set<MedicineMedAllergen> medAllergens = medicine.getMedicineMedAllergens();
        for (MedicineMedAllergen medAllergen : medAllergens) {
            if (NeoUtil.isExist(medAllergen) && !NeoUtil.inlist(medAllergen.getMedAllergen().getName(), drugBillNeoDTO.getAllergicmeds())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(medAllergen.getMedAllergen().getName());
                nodeNeoDTO.setTermtype(Constants.yaoping);
                drugBillNeoDTO.getAllergicmeds().add(nodeNeoDTO);
            }
        }


        Set<MedicineDisease> diseases = medicine.getMedicineDisease();
        for (MedicineDisease disease : diseases) {
            if (NeoUtil.isExist(disease) && !NeoUtil.inlist(disease.getDisease().getName(), drugBillNeoDTO.getDisease())) {
                NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
                nodeNeoDTO.setName(disease.getDisease().getName());
                nodeNeoDTO.setTermtype(Constants.disease);
                drugBillNeoDTO.getDisease().add(nodeNeoDTO);
            }
        }

        return drugBillNeoDTO;
    }

/*
    public NodeNeoDTO fillNodeNeoDTO(Age age) {
        NodeNeoDTO nodeNeoDTO = new NodeNeoDTO();
        Integer range;
        BigDecimal max, min;

        nodeNeoDTO.setName(age.getName());

        max = (age.getMaxval()==null)?null:new BigDecimal(age.getMaxval());
        nodeNeoDTO.setMax(max);

        min = (age.getMinval()==null)?null:new BigDecimal(age.getMinval());
        nodeNeoDTO.setMin(min);

        range = (age.getRange()==null)?0:age.getRange();
        nodeNeoDTO.setValType(range);

        nodeNeoDTO.setName(age.getTermname());

        return nodeNeoDTO;
    }
*/


    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getMedicineBill(List<Map<String, String>> medslist, MedicineRepository medicineRepository,
                                            MedRegNameRepository medRegNameRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        List<MedRegName> medRegNames;
        String medname;

        for (Map<String, String> medsterm : medslist) {
            for (String term : medsterm.keySet()) {
                billNeoDTO = new BillNeoDTO();
                medname = medsterm.get(term);
                billNeoDTO.setName(term);
                billNeoDTO.setStandname(medname);

                drugBillNeoDTO = new DrugBillNeoDTO();

                List<String> medRegNamels = medicineRepository.findMedRegNamebyMedicine(medname);

                for (String mrnames : medRegNamels) {
                    medRegNames = medRegNameRepository.findByNameIs(mrnames);

                    if (ListUtil.isNotEmpty(medRegNames)) {
                        MedRegNameNode medRegNameNode = new MedRegNameNode();
                        for (MedRegName medrn : medRegNames) {
                            drugBillNeoDTO = medRegNameNode.MedRegNametoDrugDTO(medrn, drugBillNeoDTO);
                        }
                    }
                }

                billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

                billNeoDTOs.add(billNeoDTO);

            }
        }


        return billNeoDTOs;
    }


    /**
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicineList(String name, MedicineRepository medicineRepository, BaseNodeRepository baseNodeRepository) {
        String nodename;

        List<DrugInfoDTO> medslist = new ArrayList<>();
        DrugInfoDTO drugInfoDTO;

        List<Medicine> medicines = medicineRepository.index(name, name, 100);

        if (ListUtil.isNotEmpty(medicines) && medicines.size() > 100) {
            medicines = medicines.subList(0, 100);
        }

        for (Medicine med : medicines) {
            drugInfoDTO = new DrugInfoDTO();
            nodename = med.getName();

            drugInfoDTO.setName(nodename);
            medslist.add(drugInfoDTO);
        }


        return medslist;
    }
}

