package com.diagbot.repository;

import com.diagbot.entity.BaseNodeInfo;
import com.diagbot.entity.node.PacsName;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


public interface PacsNameRepository extends Neo4jRepository<PacsName, Long> {

    List<PacsName> findByNameIs(String name);

    List<PacsName> findByNameContaining(String name);

    List<PacsName> findByNameIn(List<String> names);

    Page<PacsName> findByNameContainingIgnoreCaseOrPycodeContainingIgnoreCase(String name, String pycode, Pageable pageable);

    @Query("MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) = toLower($name)  OR  toLower(n.`拼音编码`) = toLower($pycode))\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "UNION\n" +
            "MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) STARTS WITH toLower($name) OR toLower(n.`拼音编码`) STARTS WITH toLower($pycode))\n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "UNION \n" +
            "MATCH (n:`辅助检查名称`) WHERE n.`静态知识标识` = $isKL  \n" +
            "AND (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<PacsName> staticKnowledgeIndex(@Param("isKL") Integer isKL, @Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);

    @Query("MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) = toLower($name) \n" +
            "OR toLower(n.`拼音编码`) = toLower($pycode)  \n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) starts with toLower($name) \n" +
            "OR toLower(n.`拼音编码`) starts with toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n" +
            "union\n" +
            "MATCH (n:`辅助检查名称`) WHERE toLower(n.`name`) CONTAINS toLower($name) \n" +
            "OR toLower(n.`拼音编码`) CONTAINS toLower($pycode)  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN n,[ (n)-[r]->(m) | [ r, m ] ], ID(n)\n")
    List<PacsName> index(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);

    //检索不区分辅助检查名称和辅助检查子项目名称
    @Query("MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称) \n" +
            "AND (toLower(n.`name`) = toLower($name) OR toLower(n.`拼音编码`) = toLower($pycode))  \n" +
            "RETURN labels(n) as labels,n.name as name\n" +
            "union\n" +
            "MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "AND (toLower(n.`name`) starts with toLower($name) OR toLower(n.`拼音编码`) starts with toLower($pycode))  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN labels(n) as labels,n.name as name\n" +
            "union\n" +
            "MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "AND (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN labels(n) as labels,n.name as name\n")
    List<BaseNodeInfo> indexMultipleLabel(@Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);

    @Query("MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "AND n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) = toLower($name)  OR  toLower(n.`拼音编码`) = toLower($pycode))\n" +
            "RETURN labels(n) as labels,n.name as name\n" +
            "UNION\n" +
            "MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "AND n.`静态知识标识` = $isKL \n" +
            "AND (toLower(n.`name`) STARTS WITH toLower($name) OR toLower(n.`拼音编码`) STARTS WITH toLower($pycode))\n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN labels(n) as labels,n.name as name\n" +
            "UNION \n" +
            "MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "AND n.`静态知识标识` = $isKL  \n" +
            "AND (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "WITH n SKIP 0 LIMIT $size\n" +
            "RETURN labels(n) as labels,n.name as name\n")
    List<BaseNodeInfo> staticKnowledgeIndexMultipleLabel(@Param("isKL") Integer isKL, @Param("name") String name, @Param("pycode") String pycode, @Param("size") Integer size);

    @Query("MATCH (n) WHERE (n :辅助检查名称 OR n :辅助检查子项目名称)\n" +
            "and n.`name` IN  $names  \n" +
            "RETURN labels(n) as labels,n.name as name\n")
    List<BaseNodeInfo> findByNameInMultipleLabel(@Param("names") List<String> names);
}