package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.entity.relationship.*;
import com.diagbot.util.ListUtil;
import com.diagbot.util.NeoUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PacsSubNameNode {


    public PacsBillNeoDTO PacstoPACSDTO(PacsSubName pacs) {
        PacsBillNeoDTO pacsBillNeoDTO = new PacsBillNeoDTO();
        NodeInfo nodeInfo;


        PacsSubNameGender pacsSubNameGender = pacs.getPacsSubNameGender();
        if (NeoUtil.isExist(pacsSubNameGender)) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsSubNameGender.getGender().getName());
            nodeInfo.setTypeval(Constants.gender);
            pacsBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameClinicalFinding> clinicalfindings = pacs.getPacsSubNameClinicalfindings();
        for (PacsSubNameClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getClinicalFinding().getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getClinicalFinding().getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getClinicalFinding().getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getClinicalFinding().getUnit());
            nodeInfo.setRange(clinicalfinding.getClinicalFinding().getRange());
            nodeInfo.setTypeval(Constants.clinicalfinding);
            pacsBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameVital> vitals = pacs.getPacsSubNameVitals();
        for (PacsSubNameVital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getVital().getName();
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getVital().getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getVital().getMaxval()));
            nodeInfo.setUnit(vital.getVital().getUnit());
            nodeInfo.setRange(vital.getVital().getRange());
            nodeInfo.setTypeval(Constants.vital);
            pacsBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameLis> liss = pacs.getPacsSubNameLis();
        for (PacsSubNameLis lis : liss) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getLis().getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getLis().getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getLis().getMaxval()));
            nodeInfo.setUnit(lis.getLis().getUnit());
            nodeInfo.setRange(lis.getLis().getRange());
            nodeInfo.setTypeval(Constants.lis);
            pacsBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNamePacsDescribe> conflict_pacsdesc = pacs.getPacsSubNamePacsdescribes();
        for (PacsSubNamePacsDescribe pacsdesc : conflict_pacsdesc) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsdesc.getPacsDescribe().getName());
            nodeInfo.setTypeval(Constants.pacsdesc);
            pacsBillNeoDTO.getPacsDesc().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameGroup> groups = pacs.getPacsSubNameGroups();
        for (PacsSubNameGroup group : groups) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getGroup().getName());
            nodeInfo.setTypeval(Constants.group);
            pacsBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameOralMedicine> conflict_medicines = pacs.getPacsSubNameOralmedicines();
        for (PacsSubNameOralMedicine conflict_medicine : conflict_medicines) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_medicine.getOralMedicine().getName());
            nodeInfo.setTypeval(Constants.yaoping);
            pacsBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameDisease> diseases = pacs.getPacsSubNameDiseases();
        for (PacsSubNameDisease disease : diseases) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getDisease().getName());
            nodeInfo.setTypeval(Constants.disease);
            pacsBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameMedAllergen> medAllergens = pacs.getPacsSubNameMedallergens();
        for (PacsSubNameMedAllergen medAllergen : medAllergens) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(medAllergen.getMedAllergen().getName());
            nodeInfo.setTypeval(NeoUtil.getMedType(medAllergen.getMedAllergen().getName()));
            pacsBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameAllergen> allergens = pacs.getPacsSubNameAllergens();
        for (PacsSubNameAllergen allergen : allergens) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(allergen.getAllergen().getName());
            nodeInfo.setTypeval(Constants.yaoping); // 过敏原归类成药物过敏原，与模型提取匹配
            pacsBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsSubNameConflictDevice> conflictDevices = pacs.getPacsSubNameConflictdevices();
        for (PacsSubNameConflictDevice conflictDevice : conflictDevices) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflictDevice.getConflictDevice().getName());
            nodeInfo.setTypeval(Constants.conflictdevice);
            pacsBillNeoDTO.getMeddevice().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        return pacsBillNeoDTO;
    }


    /**
     * 处理辅检子项目开单合理性
     */
    public List<BillNeoDTO> getSubPacsBill(List<Map<String, String>> pacslist, PacsSubNameRepository pacsSubRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        List<PacsSubName> pacslt;
        String pacsname;

        for (Map<String, String> pacsterm : pacslist) {
            for (String term : pacsterm.keySet()) {
                billNeoDTO = new BillNeoDTO();
                pacsname = pacsterm.get(term);
                billNeoDTO.setName(term);
                billNeoDTO.setStandname(pacsname);

                pacsBillNeoDTO = new PacsBillNeoDTO();

                pacslt = pacsSubRepository.findByNameIs(pacsname);

                if (pacslt != null && pacslt.size() > 0) {
                    pacsBillNeoDTO = PacstoPACSDTO(pacslt.get(0));
                    billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

                    billNeoDTOs.add(billNeoDTO);
                }
            }
        }

        return billNeoDTOs;
    }


    /**
     * 获取辅检子项目名称列表
     */
    public List<String> getPACSSubNameList(String name, PacsSubNameRepository pacsRepository) {
        String pacsname;

        List<String> pacslist = new ArrayList<>();

        List<PacsSubName> pacssublt = pacsRepository.index(name, name, 100);

        if (ListUtil.isNotEmpty(pacssublt) && pacssublt.size() > 100) {
            pacssublt = pacssublt.subList(0, 100);
        }

        for (PacsSubName pacs : pacssublt) {
            pacsname = pacs.getName();
            if (!pacslist.contains(pacsname)) {
                pacslist.add(pacsname);
            }
        }

        return pacslist;
    }
}






