package com.diagbot.rule;

import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.NeoEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.model.entity.Negative;
import com.diagbot.util.CatalogueUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @description: 通用规则提取
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class CommonRule {

    /**
     * 比较界面元素是否在图谱中存在（匹配名称）
     *
     * @param neoList       图谱结构
     * @param inputLises    界面元素
     * @param billNeoMaxDTO 开单项名称，开单项标准名称
     * @return
     */
    public <T> void compareNameWithBill(List<NodeNeoDTO> neoList, List<T> inputLises, BillNeoMaxDTO billNeoMaxDTO, List<BillMsg> billMsgList, String type) {
        Map<String, String> old_stand = new HashMap<>();
        if (ListUtil.isNotEmpty(neoList) && ListUtil.isNotEmpty(inputLises)) {
            List<String> dl = neoList.stream().map(x -> x.getName()).collect(Collectors.toList());
            for (T d : inputLises) {
                Negative val = (Negative) CoreUtil.getFieldValue(d, "negative");
                if (val == null) {
                    String c = (String) CoreUtil.getFieldValue(d, "standName");
                    String c_name = (String) CoreUtil.getFieldValue(d, "name");
                    if (StringUtils.isNotBlank(c) && dl.contains(c)) {
                        old_stand.put(c_name, c);
                    }
                }
            }
            if (old_stand.size() > 0) {
                for (Map.Entry<String, String> d : old_stand.entrySet()) {
                    if ("禁忌手术".equals(type)) {
                        BillMsg commonBillMsg = MsgUtil.getCommonSurgeryMsg(
                                billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                d.getKey(), type, billNeoMaxDTO.getType());
                        billMsgList.add(commonBillMsg);
                    } else {
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                                d.getKey(), type, billNeoMaxDTO.getType());
                        billMsgList.add(commonBillMsg);
                    }
                }
            }
        }
    }

    /**
     * 比较界面元素是否在图谱中存在（匹配名称）
     *
     * @param neoList       图谱结构
     * @param input         界面元素
     * @param billNeoMaxDTO 开单项名称，开单项标准名称
     * @return
     */
    public <T> void compareItemWithBill(List<NodeNeoDTO> neoList, List<T> input, BillNeoMaxDTO billNeoMaxDTO, List<BillMsg> billMsgList, String type) {
        Map<String, String> old_stand = new HashMap<>();
        if (ListUtil.isNotEmpty(neoList) && ListUtil.isNotEmpty(input)) {
            for (T d : input) {
                String c = (String) CoreUtil.getFieldValue(d, "uniqueName");
                String c_name = (String) CoreUtil.getFieldValue(d, "name");
                for (NodeNeoDTO nodeNeoDTO : neoList) {
                    if (CoreUtil.compareNodeName(nodeNeoDTO, c)) {
                        old_stand.put(c_name, c);
                    }
                }
            }
            if (old_stand.size() > 0) {
                for (String key : old_stand.keySet()) {
                    BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                            billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                            key, type, billNeoMaxDTO.getType());
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }

    /**
     * 24小时重复开单总入口
     *
     * @param wordCrfDTO
     * @param billMsgList
     */
    public void repeat24Bill(WordCrfDTO wordCrfDTO, List<BillMsg> billMsgList) {
        repeat24BillWithType(billMsgList, wordCrfDTO.getLisOrder(), TypeEnum.lis.getName()); // 化验重复开单
        repeat24BillWithType(billMsgList, wordCrfDTO.getPacsOrder(), TypeEnum.pacs.getName()); // 辅检重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getDrugOrder(), TypeEnum.drug.getName()); // 药品重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getOperationOrder(), TypeEnum.operation.getName()); // 手术重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getTransfusionOrder(), TypeEnum.transfusion.getName()); // 输血重复开单
    }

    /**
     * 互斥开单项
     *
     * @param wordCrfDTO
     * @param billMsgList
     */
    public void exclusionBill(List<BillNeoMaxDTO> billNeoMaxDTOList, WordCrfDTO wordCrfDTO, List<BillMsg> billMsgList) {
        // 辅检开单项互斥
        List<Pacs> pacsOrder = wordCrfDTO.getPacsOrder();
        Set<String> set = new LinkedHashSet<>();    // A与B不宜同时进行, B与A不宜同时进行 只能提示一个
        if (ListUtil.isNotEmpty(pacsOrder) && pacsOrder.size() > 1) {
            // 不同的辅检项目可能对应同一个uniqueName，提示显示多条
            Map<String, List<Pacs>> map = EntityUtil.makeEntityListMap(pacsOrder, "uniqueName");
            for (BillNeoMaxDTO billNeoMaxDTO : billNeoMaxDTOList) {
                List<NodeNeoDTO> nodeList = billNeoMaxDTO.getPacsOrder();
                if (ListUtil.isNotEmpty(nodeList)) {
                    for (NodeNeoDTO nodeNeoDTO : nodeList) {
                        List<Pacs> orginNameList = map.get(nodeNeoDTO.getName());
                        if (map.get(nodeNeoDTO.getName()) != null) {
                            for (Pacs pacs : orginNameList) {
                                String orginName = pacs.getName();
                                if (!set.contains(billNeoMaxDTO.getOrderName() + "******" + orginName)) {
                                    set.add(billNeoMaxDTO.getOrderName() + "******" + orginName);
                                    set.add(orginName + "******" + billNeoMaxDTO.getOrderName());
                                    BillMsg commonBillMsg = MsgUtil.getBillExclusionMsg(
                                            billNeoMaxDTO.getOrderName(), orginName,
                                            billNeoMaxDTO.getOrderName() + "，" + orginName,
                                            NeoEnum.exclusion.getName());
                                    billMsgList.add(commonBillMsg);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 24小时重复开单入口
     *
     * @param billMsgList
     * @param itemList
     * @param type
     */
    public <T> void repeat24BillWithType(List<BillMsg> billMsgList, List<T> itemList, String type) {
        if (ListUtil.isEmpty(itemList)) {
            return;
        }
        Map<String, List<T>> map = EntityUtil.makeEntityListMap(itemList, "name");
        for (String key : map.keySet()) {
            List<T> items = map.get(key);

            // 个数大于2个才比较
            if (ListUtil.isNotEmpty(items) && items.size() > 1) {
                // items 按照时间排序
                Collections.sort(items, new Comparator<T>() {
                    @Override
                    public int compare(T o1, T o2) {
                        String o1Str = (String) CoreUtil.getFieldValue(o1, "dateValue");
                        String o2Str = (String) CoreUtil.getFieldValue(o2, "dateValue");
                        if (StringUtil.isBlank(o1Str) || StringUtil.isBlank(o2Str)) {
                            return -1;
                        } else {
                            return o1Str.compareTo(o2Str);
                        }
                    }
                });

                // 比较时间是否在24小时内
                List<Date> dateList = new ArrayList<>();
                for (T it : items) {
                    String dateValue = (String) CoreUtil.getFieldValue(it, "dateValue");
                    if (StringUtil.isNotBlank(dateValue)) {
                        Date cur = CatalogueUtil.parseStringDate(dateValue);
                        // 如果为null，说明日期格式出错，不比较
                        if (cur != null) {
                            if (ListUtil.isNotEmpty(dateList)) {
                                Date last = dateList.get(dateList.size() - 1);
                                if (!CatalogueUtil.compareTime(last, cur, 60L * 24)) {
                                    String name = (String) CoreUtil.getFieldValue(it, "name");
                                    String uniqueName = (String) CoreUtil.getFieldValue(it, "uniqueName");
                                    BillMsg commonBillMsg = MsgUtil.getBillMsg24Repeat(
                                            name, uniqueName, name, type);
                                    billMsgList.add(commonBillMsg);
                                    break;
                                } else {
                                    dateList.add(cur);
                                }
                            } else {
                                dateList.add(cur);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 高风险药品、手术
     *
     * @param highRiskNeoDTO
     * @param highRiskList
     */
    public void highRisk(HighRiskNeoDTO highRiskNeoDTO, List<BillMsg> highRiskList) {
        BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(highRiskNeoDTO.getName(),
                highRiskNeoDTO.getHighLevel(), highRiskNeoDTO.getType());
        highRiskList.add(billMsg);
    }

    /**
     * fa'zha
     *
     * @param highRiskNeoDTO
     * @param highRiskList
     */
    public void highRiskComplex(NodeNeoDTO nodeNeoDTO, List<BillMsg> highRiskList, HighRiskNeoDTO highRiskNeoDTO) {
        BillMsg billMsg = MsgUtil.getComplexOperationMsg(nodeNeoDTO.getVal(), highRiskNeoDTO.getName(), nodeNeoDTO.getTermtype());
        highRiskList.add(billMsg);
    }
}
